package com.friday.tooldriver.mapper.executor;

import com.friday.operation.api.executor.dto.executor.ExecutorFilter;
import com.friday.tooldriver.po.executor.ExecutorPo;
import org.springframework.stereotype.Repository;

import java.util.List;

@Repository
public interface ExecutorMapper {
    void insertExecutor(ExecutorPo executorPo);
    void deleteExecutorById(Integer id);
    ExecutorPo selectExecutorById(Integer id);
    List<ExecutorPo> selectExecutors(ExecutorFilter filter);
    List<ExecutorPo> selectAllExecutors();
    List<ExecutorPo> selectAllRelatedServiceExecutors();
}
